#!/bin/sh
set -x

echo "S51" > /proc/gpio

#workaroud input/output error
cp /bin/busybox /var/

FW_PATH="/var/tmp/fw"
FW_BIN="$FW_PATH"/fw.bin
FW_BOOT="$FW_PATH"/boot.bin
#FW_CFG="$FW_PATH"/config.json
FMW_BOARD_TYPE="$(cat $FW_PATH/board)"
MODEL_NAME="$(factory board)"
HW_REVISION="$(factory hw)"
INDICATION_INFO="$(cat /proc/indication | grep power | sed -e 's/power: .*/power/g')"

echo $MODEL_NAME
echo $HW_REVISION

UpgradeComplite=0
UpgradeError=1
UpgradeInProgress=2

#if [ "$FMW_BOARD_TYPE" != "$MODEL_NAME" ]; then
#    echo "UPDATE FAIL: board is $MODEL_NAME, but firmware is $FMW_BOARD_TYPE"
#    echo "S50" > /proc/gpio
#    exit 2
#fi

chmod 777 "$FW_PATH"/upgrade-bin
#new_mib_lib="$FW_PATH/lib/apmib/"
#property_lib="$FW_PATH/lib/property/"

UPGRADE_OPTS=""

if [[ "$MODEL_NAME" == "RR-10V" && "$FMW_BOARD_TYPE" != "RR-10V" ]]; then
    echo "UPGRADE FAIL: Image just for RR-10V" > /dev/console
    echo "S50" > /proc/gpio
    exit 7
fi

if [[ "$MODEL_NAME" != "RR-10V" && "$FMW_BOARD_TYPE" == "RR-10V" ]]; then
    echo "UPGRADE FAIL: Image just for RR-1X" > /dev/console
    echo "S50" > /proc/gpio
    exit 7
fi

if [ -f "$FW_BOOT" ]; then
    md5_cur=$(busybox md5sum "$FW_PATH"/boot.bin | cut -d' ' -f 1)
    md5_img=$(cat "$FW_PATH"/boot.md5)
    if [ "$md5_cur" != "$md5_img" ]; then
        echo "UPGRADE FAIL: Incorrect boot CRC"
	    echo "S50" > /proc/gpio
        exit 3
    fi
    UPGRADE_OPTS="$UPGRADE_OPTS -b $FW_BOOT"

    echo "Upgrade boot started"
    "$FW_PATH"/upgrade-bin $UPGRADE_OPTS
    RETVAL=$?

    #update board info
    [ "$RETVAL" == "$UpgradeComplite" ] && board_info -l "$md5_img"
fi

if [ -f $FW_BIN ]; then
    md5_cur=$(busybox md5sum "$FW_PATH"/fw.bin | cut -d' ' -f 1)
    md5_img=$(cat "$FW_PATH"/fw.md5)
    if [ "$md5_cur" != "$md5_img" ]; then
        echo "UPGRADE FAIL: Incorrect fw CRC"
	    echo "S50" > /proc/gpio
        exit 4
    fi
    UPGRADE_OPTS="$UPGRADE_OPTS -f $FW_BIN"
fi

echo "Upgrade firmware started"

if [[ "$MODEL_NAME" == "RR-10" ]]
then
    if [[ "$HW_REVISION" == "1v1" || "$HW_REVISION" == "1v2" || "$HW_REVISION" == "1v3" ]]
    then
        if [[ "$INDICATION_INFO" == "power" ]]
        then
            echo "pr24" > /proc/gpio
        else
            echo "r14" > /proc/gpio
        fi
    else
        echo "Failed to identify revesion for RR-10" > /dev/console
    fi
fi

if [[ "$MODEL_NAME" == "RR-10 rev.B" ]]
then
    if [[ "$HW_REVISION" == "2v0" || "$HW_REVISION" == "2v1" || "$HW_REVISION" == "2v2" ]]
    then
        echo "pr14" > /proc/gpio
        echo "sn4"  > /proc/gpio
    else
        echo "Failed to identify revesion for RR-10 rev.B" > /dev/console
    fi
fi

if [[ "$MODEL_NAME" == "RR-10V" ]]
then
    if [[ "$HW_REVISION" == "2v0" || "$HW_REVISION" == "2v1" || "$HW_REVISION" == "2v2" ]]
    then
        echo "pr14" > /proc/gpio
        echo "sn4"  > /proc/gpio

	[ -f "$FW_PATH"/voice-rootfs.jffs2 ] && {
	    umount /usr/share/voice/
	    cat "$FW_PATH"/voice-rootfs.jffs2 > /dev/mtdblock7
	    echo "UPGRADE OK: upgrade voice-rootfs.jffs2 image"
        }
    else
        echo "Failed to identify revesion for RR-10V" > /dev/console
    fi
fi

if [[ "$MODEL_NAME" == "RR-11" ]]
then
    if [[ "$HW_REVISION" == "1v0" || "$HW_REVISION" == "1v1" || "$HW_REVISION" == "1v2" || "$HW_REVISION" == "1v3" ]]
    then
        echo "pr14" > /proc/gpio
        echo "sn4"  > /proc/gpio
        echo "wn4"  > /proc/gpio
    else
        echo "Failed to identify revesion for RR-11" > /dev/console
    fi
fi

"$FW_PATH"/upgrade-bin $UPGRADE_OPTS
RETVAL=$?

#update board info
#[ "$RETVAL" == "$UpgradeComplite" ] && { 

#}

if [ "$RETVAL" != "$UpgradeComplite" ]; then
    if [ "$RETVAL" = "$UpgradeInProgress" ]; then
        echo "UPGRADE FAIL: upgrade process is already in progress"
        echo "R" > /proc/gpio # restore prev led state
        exit 5
    fi
    echo "UPGRADE FAIL: Error at read/write operation ($RETVAL != $UpgradeComplite)"
    echo "S50" > /proc/gpio
    echo "R" > /proc/gpio # restore prev led state
    exit 6
fi


ver_img=$(cat "$FW_PATH"/version | head -n1)
ln -s "$FW_PATH"/upgrade-bin "$FW_PATH"/board_info
ln -s "$FW_PATH"/upgrade-bin "$FW_PATH"/update_mib
"$FW_PATH"/board_info -w "$ver_img"
"$FW_PATH"/board_info -d "$md5_img"
"$FW_PATH"/update_mib

echo "UPGRADE SUCCESSFULL"

# for old rom compatibility
#cat /bin/upgrade_fw.sh | grep "# version 1.01" > /dev/null
if [ $? != "0" ]; then
    reboot

    #workaroud input/output error
    /var/busybox reboot
    if [ $? != "0" ]; then
        killall -9 timelycheck watchdog
    fi
fi
echo "S50" > /proc/gpio
exit $UpgradeComplite
